#
#    Copyright (c) 1996-2000 Brocade Communications Systems, Inc.
#    All rights reserved.
#
#    File name:   diagcommon.sh
#    Module name: diag/scripts
#
#    This file contains several useful functions used by the
#    diagnostics user command scripts.  They are stuck here
#    to save some disk image space.  To optimize execution time
#    copy this file into each of the command scripts instead.
#

_default_home="/fabos"                 # FABOSHOME default
_cfg_cmd="config"		      # config program
_def_slot=0

# definitions that must match src/include/config/resource.h 
# define Resource* (line 68-line 74)
INTEGER=2			      # config modes
STRING=5
HEX=6
# definitions that must match src/include/em/em_if.h ledState_e
LED_DEFAULT=0
LED_SOLID_OFF=1
LED_SOLID_ON=2
LED_FAST_BLINK=3
LED_SLOW_BLINK=4
##################
# Utilities
#

# error
# $1 error string
err () {
	/bin/echo "" >&2
	/bin/echo "Error -- $1" >&2
	if [ -n "$syntax" ] ; then
		eval /bin/echo Syntax: $syntax >&2
	fi
}

# commandPath
#
# Get full path to FABOS command
# $1 command name
#
commandPath () {
	# First try fabos/sbin/command
	if [ "$FABOSHOME" = "" ] ; then
		_cmd="$_default_home/sbin/$1"
	else
		_cmd="$FABOSHOME/sbin/$1"
	fi
	if [ -x $_cmd ] ; then
		/bin/echo $_cmd
		return 0
	fi

	# No luck -- try fabos/bin/command
	if [ "$FABOSHOME" = "" ] ; then
		_cmd="$_default_home/bin/$1"
	else
		_cmd="$FABOSHOME/bin/$1"
	fi
	if [ -x $_cmd ] ; then
		/bin/echo $_cmd
		return 0
	fi

	# Still no luck try fabos/command
	if [ "$FABOSHOME" = "" ] ; then
		_cmd="$_default_home/$1"
	else
		_cmd="$FABOSHOME/$1"
	fi
	if [ -x $_cmd ] ; then
		/bin/echo $_cmd
		return 0
	fi

	# Still no luck try fabos/share
	if [ "$FABOSHOME" = "" ] ; then
		_cmd="$_default_home/share/$1"
	else
		_cmd="$FABOSHOME/share/$1"
	fi
	if [ -x $_cmd ] ; then
		/bin/echo $_cmd
		return 0
	fi

	# Last but not least try plain old command.
	_cmd="$1"
	if [ -x $_cmd ] ; then
		/bin/echo $_cmd
		return 0
	fi

	# Ok we give up now.
	err "Could not find: $_cmd"
	return 3

}

# setConfig()
#
# Set configuration variable value
# $1 = config string to set
# $2 = config mode
# $3 = new value
#
setConfig () {
	_cmd=`commandPath $_cfg_cmd`
	if [ $? != 0 ] ; then exit 3 ; fi
	if { $_cmd set "$1" "$2" "$3"; } ; then
		return 0
	else
		err "setConfig failed"
		exit 3
	fi
}

# commitConfig()
#
# Push configuration changes to kernel.
#  -- no inputs required.
#
commitConfig () {
	_cmd=`commandPath $_cfg_cmd`
	if [ $? != 0 ] ; then exit 3 ; fi
	if { $_cmd commit; } ; then
	    /bin/echo "Config commit Succeeded" >&2
	    return 0
	else
	    err "commitConfig failed"
	    exit 3
	fi
}

# updateConfig() 
# 
# Commit configuration changes to NV storage. 
#  -- no inputs required. 
# 
updateConfig () { 
	_cmd=`commandPath $_cfg_cmd` 
	if [ $? != 0 ] ; then exit 3 ; fi
	for _try in 1 2 3 4 5 6 7 8 9 10; do
	    if { $_cmd update 0 0 2>/dev/null | /bin/grep --quiet "done" ; } ; then 
		/bin/echo "Config update Succeeded" >&2
		return 0 
	    fi
	    if [ $_try -ne 10 ] ; then
		/bin/echo "Config update failed, retrying" >&2
		/bin/sleep 3
	    fi
	done;

	# Should not get here
	err "Config update failed" 
      	exit 3 
}
 
# ConfigSetCheck()
#
# Check if a config variable has been set
# $1 = config string to set
# $2 = config mode
#
ConfigSetCheck () {
	_cmd=`commandPath $_cfg_cmd`
	if [ $? != 0 ] ; then exit 3 ; fi
	_val=`$_cmd get "$1" "$2" 2>/dev/null`
	if [ -z "$_val" ] ; then
		return 1
	fi
	return 0
}

# getConfig()
#
# Set configuration variable value
# $1 = config string to set
# $2 = config mode
# $3 = default
#
getConfig () {
	_cmd=`commandPath $_cfg_cmd`
	if [ $? != 0 ] ; then exit 3 ; fi
	_val=`$_cmd get "$1" "$2" 2>/dev/null`
	if [ -z "$_val" ] ; then
		_val="$3"
	fi
	/bin/echo $_val
	return 0
}

# testConfig()
#
# Set configuration variable value
# $1 = config string to test
# $2 = config mode
#
testConfig () {
	_cmd=`commandPath $_cfg_cmd`
	if [ $? != 0 ] ; then exit 3 ; fi
	_val=`$_cmd get "$1" $2 2>/dev/null`
	if [ -z "$_val" ] ; then
		/bin/echo "0"
	else
		/bin/echo "1"
	fi
	return 0
}

# checkForShow()
#
# Display current setting and bail if this is a show
# command.  Also check for -inc and increment the current
# setting but only if the config_mode is not STRING and
# the value is not boolean.
#
# $1 = config_name
# $2 = config_string
# $3 = config_mode
# $4 = config_default
# $5 = argument to check. (must be last)

checkForShow () {
	_s=`/usr/bin/expr "$5" : "[-]*show"`
	if { [ -z "$5" ] || [ "0" -ne "$_s" ] ; } ; then
		_val=`getConfig $2 $3 $4`
		if [ $? != 0 ] ; then exit 3 ; fi
		if { [ -n "$bool_true" ] && [ -n "$bool_false" ] ; } ; then
			if [ "$_val" -ne 0 ] ; then
				_res=$bool_true
			else
				_res=$bool_false
					fi
			/bin/echo "$1 is $_val ($_res)".
		else
			/bin/echo "$1 is $_val".
		fi

		exit 0
	else
		_s=`/usr/bin/expr "$5" : "[-]*inc"`
		if { [ "$3" != "$STRING" ] && [ "0" -ne "$_s" ] && \
				 [ -z "$bool_true" ] && [ -z "$bool_false" ] ; }
		then
			_val=`getConfig $2 $3 $4`
			if [ $? != 0 ] ; then exit 3 ; fi
			_val=`/usr/bin/expr $_val + 1`
			setConfig $2 $3 $_val
			/bin/echo "$1 is now ${_val}."
			exit 0
		fi
		return 0
	fi
}

# getValue()
#
# Converts input string into number or generates an
# error.  Special keywords [-]enable (1) [-]disable (0)
# [-]on (1) and [-off] (0) are also supported for
# boolean conditions
#
# $1 = parameter to check

getValue () {

	_s=`/usr/bin/expr "$1" : "[-]*enable"`
	if [ "0" -ne "$_s" ]; then
		/bin/echo "1"
		return 0
	fi

	_s=`/usr/bin/expr "$1" : "[-]*on"`
	if [ "0" -ne "$_s" ]; then
		/bin/echo "1"
		return 0
	fi

	_s=`/usr/bin/expr "$1" : "[-]*disable"`
	if [ "0" -ne "$_s" ]; then
		/bin/echo "0"
		return 0
	fi

	_s=`/usr/bin/expr "$1" : "[-]*off"`
	if [ "0" -ne "$_s" ]; then
		/bin/echo "0"
		return 0
	fi

	_n=`/bin/echo "$1" | \
		/usr/bin/wc -c | \
		/usr/bin/tr -s " " " " | \
		/usr/bin/cut -d" " -f2`
	_n=`/usr/bin/expr $_n - 1`
	_s=`/usr/bin/expr "$1" : "[-][0-9]*"`
	_t=`/usr/bin/expr "$1" : "[0-9]*"`
	if [ "$_n" -eq "$_s" -o  "$_n" -eq "$_t" ]; then
		/bin/echo $1
		return 0
	fi

	err "Numeric input required"
	/bin/echo "0"
	exit 3
}

##################################
# NAME
#    enableSlotProc - enable /proc/fabos/blade/slot entries
#
# SYNOPSIS
#    enableSlotProc slot_no
#
# DESCRIPTION
#    
#    1) check slot existence
#    2) get proc entries by calling debugentryon
#    3) varify the exection by checking its return status
#	if fail, exit and tell the user to relinguish others
#	if success, continue
#
# OPERANDS
#    slot_no
##################################
function enableSlotProc() {
	if [ ! -e /proc/fabos/blade/$1 ] ; then
	    err "Slot: $slot not found"
	    exit 3
	fi
	debugentryon --slot $1 > /dev/null 2>&1
	if [ "0" -ne "$?" ] ; then
	    if [ `isMultiBlade` = TRUE ]
	    then
		/bin/echo "Slot$1: not enough proc entires, relinguish by running "debugEntryOff" first"
		/bin/echo "       Usage: debugEntryOff --slot x"
	    else
		/bin/echo "Not enough proc entires, relinguish by running "debugEntryOff" first"
		/bin/echo "       Usage: debugEntryOff"
	    fi
	      exit 3
	 fi
}

##################################
# NAME
#    disableSlotProc - disable /proc/fabos/blade/slot entries
#
# SYNOPSIS
#    disableSlotProc slot_no
#
# DESCRIPTION
#    
#    release previously enabled proc entries.
#
# OPERANDS
#    slot_no
##################################
function disableSlotProc() {
	 # relinguish the proc entries
	 debugentryoff --slot $1 > /dev/null 2>&1
}

#
# Begin burnin script support
# Bill Lewis 7/24/2001
#
##################################
#PRIVATE FUNCTIONS
##################################
##################################
# This function import_burnin_variables() is intended for
# internal use only. It ensures that each PARAMDEF entry exists
# in the config database.
#
# It gets the variable name from field 2 of the file, and the 
# default value from field 3. If the item does not exist, then it is 
# created, and given the default value. Otherwise, the config 
# database value is left untouched.
# $1 = PARAMDEF to parse
# space seperated list
#  field 1 = "PARAMDEF"
#  field 2 = varaible name
#  field 3 = default value
#  field 4 = mode: $INTEGER, $STRING
#  field 5 = minimum value
#  field 6 = maximum value
#  field 7 = description

##################################
import_burnin_variables()
{
	_param_list=`get_param_list`
	export _num_params=`/bin/echo $_param_list | \
				 /usr/bin/tr -s " " "_" | \
				 /usr/bin/tr -s "@" " " | \
				 /usr/bin/wc -w`
	config_list=`get_config_list`
	_num=1
	while [ $_num -le $_num_params ]
	do
		_param=`/bin/echo $_param_list | /usr/bin/cut -d"@" -f$_num`
		_var_name=`/bin/echo $_param | /usr/bin/cut -d" " -f2`
		_dflt_val=`/bin/echo $_param | /usr/bin/cut -d" " -f3`
		_mode_val=`/bin/echo $_param | /usr/bin/cut -d" " -f4`
		_cur_val="`get_config_entry $_var_name`"
		if [ -z "$_cur_val" ]
		then
			_cur_val=$_dflt_val
			setConfig diag.$MyName.$_var_name $_mode_val "$_dflt_val"
		fi
		burninMessage "$_var_name=\""$_cur_val\"
		export `eval /bin/echo $_var_name`="`eval /bin/echo $_cur_val`"
		_num=`/usr/bin/expr $_num + 1`
	done
}

##################################
# This function valide_integer() is an internal only function.
# It determines if an integer is between two values. It
# expects the first parameter to be the value in question. The
# second value to be the minimum acceptable value, and the
# third value to be the maximum acceptable value. The min and
# max values are considered valid.
# $1 = value to test
# $2 = minimum acceptable value
# $3 = maximum acceptable value
##################################
validate_integer()
{
	_pass=TRUE
	_x=`/usr/bin/expr $1 : "\([-]*[0-9]\+\)"`
	if [ "$_x" != "$1" ]
	then
		_pass=NOTNUMBER
	else
		if [ "$2" != "NA" ]
		then
			_x=`/usr/bin/expr $2 : "\([-]*[0-9]\+\)"`
			if [ "$_x" != "$2" ]
			then
				_pass=MINNOTNUMBER
			else
				if [ "$1" -lt "$2" ]
				then
					_pass=LESSTHANMIN
				fi
			fi
		fi
		if [ "$3" != "NA" ]
		then
			_x=`/usr/bin/expr $3 : "\([-]*[0-9]\+\)"`
			if [ "$_x" != "$3" ]
			then
				_pass=MAXNOTNUMBER
			else
				if [ "$1" -gt "$3" ]
				then
					_pass=GREATERTHANMAX
				fi
			fi
		fi
	fi
	/bin/echo $_pass
}

##################################
# This function diagsetcycledefault() is for library use only.
# It processes each PARAMDEF variable, and sets each variable
# to the default value for each parameter. The config database
# is updated only if a value is modified values. If values are 
# not found in the config database, entries are inserted with 
# the default values.
#
# The format of the PARAMDEF is inturpeted as follows:
# $1 = space seperated PARAMDEF
# Field Description
# 1     PARAMDEF
# 2     Variable name
# 3     Default value for named variable
# 4     Mode of the variable ($INTEGER, $STRING)
# 5     Minimum valid value of named variable (NA if not applicable)
# 6     Maximum valid value of named variable (NA if not applicable)
# 7     Description of the effect of the named variable
#
# The configdata base is queried to determine 
# the current value. If the entry does not exist, then an 
# entry is created with the default value. The user is queried 
# for modification and the value is updated if it is not the default.
##################################
diagsetcycledefault()
{
	/bin/echo "DEFAULT	- KEYWORD	: COMMENT"
	_num=1
	while [ $_num -le $_num_params ]
	do
		_param=`/bin/echo $_param_list | /usr/bin/cut -d"@" -f$_num`
		_var_name=`/bin/echo $_param | /usr/bin/cut -d" " -f2`
		_dflt_val=`/bin/echo $_param | /usr/bin/cut -d" " -f3`
		_mode_val=`/bin/echo $_param | /usr/bin/cut -d" " -f4`
		_cmnt=`/bin/echo $_param     | /usr/bin/cut -d" " -f7-`
		_cur_val="`get_config_entry $_var_name`"
		if [ $_dflt_val != "$_cur_val" ]
		then
			/bin/echo replacing $_cur_val with default $_dflt_val
			setConfig diag.$MyName.$_var_name $_mode_val "$_dflt_val"
			_modified=TRUE
		fi
		/bin/echo " $_dflt_val	- $_var_name	: $_cmnt "
		_num=`/usr/bin/expr $_num + 1`
	done
	config_list=`get_config_list`
}

##################################
# This function diagsetcycleshow() is for library use only.
# It processes each PARAMDEF variable, and outputs the current
# setting for each variable. The config database is used to 
# get the current setting. If values are not found in the config
# database, the default value is output.
#
# The format of the PARAMDEF is inturpeted as follows:
# $1 = space seperated PARAMDEF
# Field Description
# 1     PARAMDEF
# 2     Variable name
# 3     Default value for named variable
# 4     Mode of the variable ($INTEGER, $STRING)
# 5     Minimum valid value of named variable (NA if not applicable)
# 6     Maximum valid value of named variable (NA if not applicable)
# 7     Description of the effect of the named variable
##################################
diagsetcycleshow()
{
	/bin/echo "CURRENT	- KEYWORD	: DEFAULT "
	_num=1
	while [ $_num -le $_num_params ]
	do
		_param=`/bin/echo $_param_list | /usr/bin/cut -d"@" -f$_num`
		_var_name=`/bin/echo $_param | /usr/bin/cut -d" " -f2`
		_dflt_val=`/bin/echo $_param | /usr/bin/cut -d" " -f3`
		_mode_val=`/bin/echo $_param | /usr/bin/cut -d" " -f4`
		_cmnt=`/bin/echo $_param     | /usr/bin/cut -d" " -f7-`
		_cur_val="`get_config_entry $_var_name`"
		if [ -z "$_cur_val" ]
		then
			_cur_val=$_dflt_val
		fi
		/bin/echo " $_cur_val	- $_var_name	: $_dflt_val "
		_num=`/usr/bin/expr $_num + 1`
	done
}

##################################
# This function diagsetcycleinteractive() is for library use only.
# It processes each PARAMDEF variable, and asks user for new 
# value for each parameter. The config database is updated with
# the modified values. If values are not found in the config 
# database, entries are inserted with the default values.
#
# The format of the PARAMDEF is inturpeted as follows:
# $1 = space seperated PARAMDEF
# Field Description
# 1     PARAMDEF
# 2     Variable name
# 3     Default value for named variable
# 4     Mode of the variable ($INTEGER, $STRING)
# 5     Minimum valid value of named variable (NA if not applicable)
# 6     Maximum valid value of named variable (NA if not applicable)
# 7     Description of the effect of the named variable
#
# The configdata base is queried to determine 
# the current value. If the entry does not exist, then an 
# entry is created with the default value. The user is queried 
# for modification and the value is updated based on the response.
#
# If the value is outside of the min/max limits, the user is 
# re-prompted for the value. If there have been three unsuccessful 
# attempts to modify the value, then the default value is taken, 
# and the next file is processed.
##################################
diagsetcycleinteractive()
{
	/bin/echo "CURRENT - KEYWORD	- DEFAULT	, COMMENT: "
	_num=1
	while [ $_num -le $_num_params ]
	do
		_param=`/bin/echo $_param_list | /usr/bin/cut -d"@" -f$_num`
		_var_name=`/bin/echo $_param | /usr/bin/cut -d" " -f2`
		_dflt_val=`/bin/echo $_param | /usr/bin/cut -d" " -f3`
		_mode_val=`/bin/echo $_param | /usr/bin/cut -d" " -f4`
		_cmnt=`/bin/echo $_param     | /usr/bin/cut -d" " -f7-`
		_cur_val="`get_config_entry $_var_name`"
		if [ -z "$_cur_val" ]
		then
			_cur_val=$_dflt_val
			_in_config=FALSE
		else
			_in_config=TRUE
		fi
		_ok=FALSE
		_attempts=0
		while [ $_attempts -lt 3 -a $_ok != TRUE ]
		do
			/bin/echo -n " $_cur_val - $_var_name	- $_dflt_val	, $_cmnt: "
			read _answer
			_attempts=`/usr/bin/expr $_attempts + 1`
			if [ -z "$_answer" ] 
			then
				if [ "$_in_config" = "FALSE" ]
				then
					setConfig diag.$MyName.$_var_name $_mode_val $_dflt_val
					_modified=TRUE
				fi
				_ok=TRUE
			else
				if [ $_mode_val = $INTEGER ]
				then
					_min_val=`/bin/echo $_param  | /usr/bin/cut -d" " -f5`
					_max_val=`/bin/echo $_param  | /usr/bin/cut -d" " -f6`
					_ok=`validate_integer $_answer $_min_val $_max_val`
					if [ $_ok != TRUE ]
					then
						/bin/echo $_ok
						if [ $_attempts -eq 3 ]
						then
							/bin/echo " ERROR - Wrong value entered three times, using default value $_dflt_val"
							_ok=TRUE
							_answer=$_dflt_val
						else
							/bin/echo " ERROR - Invalid value. Please specify a value between $_min_val and $_max_val inclusive"
						fi
					fi
				else
					_ok=TRUE
				fi
				if [ $_ok = TRUE ]
				then
					/bin/echo " Saving new value: $_answer"
					setConfig diag.$MyName.$_var_name $_mode_val "$_answer"
					_modified=TRUE
				fi
			fi
		done
		_num=`/usr/bin/expr $_num + 1`
	done
	config_list=`get_config_list`
}

##################################
# This function diagsetcycleparameters() is for library use only.
# It processes a list of arguments from the command line, and 
# updates the config database if the PARAMDEF exists for the 
# keyword/value pair received from the command line. The config
# database is updated with the modified values. If values are
# not found in the config database and the entry appears in the
# PARAMDEF for ther script, then entries are inserted in the
# config databasewith the specified values.
#
# This function requires the arguments top be a list of files to process.
#
# The format of the PARAMDEF is inturpeted as follows:
# $1 = space seperated PARAMDEF
# Field Description
# 1     PARAMDEF
# 2     Variable name
# 3     Default value for named variable
# 4     Mode of the variable ($INTEGER, $STRING)
# 5     Minimum valid value of named variable (NA if not applicable)
# 6     Maximum valid value of named variable (NA if not applicable)
# 7     Description of the effect of the named variable
#
# If the value is outside of the min/max limits, the update will not 
# proceed.
##################################
diagsetcycleparameters()
{
	_parameters=FALSE
	i=1
	while [ "$i" -le $_num_env ]
	do
		i=`/usr/bin/expr $i + 1`
		new_param=`/bin/echo $_env | /usr/bin/cut -d"@" -f$i`
		_key=`/bin/echo $new_param | /usr/bin/cut -d" " -f1`
		_val=`/bin/echo $new_param | /usr/bin/cut -d" " -f2-`
		old_param=`get_param_list | /bin/grep -w $_key`
		if [ ! -z "$old_param" ]
		then
			_dflt_val=`/bin/echo $old_param | /usr/bin/cut -d" " -f3`
			_mode_val=`/bin/echo $old_param | /usr/bin/cut -d" " -f4`
			if [ $_mode_val = $INTEGER ]
			then
				_min_val=`/bin/echo $old_param  | /usr/bin/cut -d" " -f5`
				_max_val=`/bin/echo $old_param  | /usr/bin/cut -d" " -f6`
				_ok=`validate_integer $_val $_min_val $_max_val`
				if [ $_ok != TRUE ]
				then
					/bin/echo " ERROR - Invalid value for $_key. Value between $_min_val and $_max_val inclusive."
				else
					/bin/echo " Setting $_key to $_val."
					setConfig diag.$MyName.$_key $_mode_val "$_val"
					_modified=TRUE
				fi
			else
				/bin/echo " Setting $_key to $_val."
				setConfig diag.$MyName.$_key $_mode_val "$_val"
				_modified=TRUE
			fi
		else
			/bin/echo Could not find PARAMDEF for $_key
		fi
	done
	_num_env=0
	config_list=`get_config_list`
	_env=""
}

##################################
# This function process_diagsetcycle() is for library use only.
# It processes the remaining command line arguments, and dispatches
# work to one of four processing routines (diagsetcycledefault,
# diagsetcycleshow, diagsetcycleparameters, diagsetcycleinteractive).
#
# Each of these processing routines can be called multiple times in
# the order specified in the command line except for the interactive
# processer. The interactive is only called if no other option is
# specified.
#
# If any of the processing routines modified the content of the
# config database during their operation, the config database is 
# committed to non-volitile storage.
##################################
process_diagsetcycle()
{
	_param_list=`get_param_list`
	export _num_params=`/bin/echo $_param_list | \
				 /usr/bin/tr -s " " "_" | \
				 /usr/bin/tr -s "@" " " | \
				 /usr/bin/wc -w`
	# Cache the config data base
	config_list=`get_config_list`
	_modified=FALSE
	_interactive=TRUE
	_parameters=FALSE
	if [ "$#" -ne 0 ]
	then
		_num_env=0
		shift
		_env=""
		while [ "$#" -gt 0 ]
		do
			_par=`/bin/echo $1 | /bin/sed "s/^-//g"`
			shift
			case "$_par" in 
			"default") if [ $_parameters = TRUE ]
				   then
					diagsetcycleparameters
				   fi
				   diagsetcycledefault
				   _interactive=FALSE;;
			"show")    if [ $_parameters = TRUE ]
				   then
					diagsetcycleparameters
				   fi
				   diagsetcycleshow
				   _interactive=FALSE;;
			*)  if [ $# -eq 0 ]
			    then
				    /bin/echo "There was no value for the keyword $_par."
			    else
				    _val=`/bin/echo $* | /usr/bin/cut -d"-" -f1`
				    _env=`/bin/echo "$_env@$_par $_val"`
				    num_args=`/bin/echo $_val | /usr/bin/wc -w`
				    while [ $num_args -gt 0 ]
				    do
					    shift
					    num_args=`/usr/bin/expr $num_args - 1`
				    done
				    _num_env=`/usr/bin/expr $_num_env + 1`
				   _interactive=FALSE
				   _parameters=TRUE
			    fi;;
			esac
		done
		if [ $_interactive = TRUE ]
		then
			diagsetcycleinteractive
		fi
		if [ $_parameters = TRUE ]
		then
			diagsetcycleparameters
		fi
		if [ "$_modified" = "TRUE" ]
		then
			/bin/echo "Committing changes to configuration"
			updateConfig
		fi
	fi
	exit 0
}

####################################
# PRIVATE STATUS AND SIGNAL HANDLING
#
# cleanup() function to process trapped signal received. 
# since the script handles two operations (diagsetcycle and burnin),
# the abort operation must be aware of which operation is active.
#
# For the diagsetcycle operation, only the exit with status is
# performed.
#
# For the burnin operation, any active child process is killed
# with the TERM signal, status, and messages are output, then the
# normal exit function is called
####################################
diagcommon_cleanup()
{
	if [ "$State" = "DIAGSETCYCLE" ]
	then
		exit $CumulativeStatus
	fi

	if [ $Child -ne 0 ]
	then
		/bin/kill -TERM $Child
		/bin/echo ""
		burninMessage "Exiting. Kill pid $Child"
	fi
	# set abort flag
	#
	State=ABORT
	burninMessage "Script has been ABORTED `/bin/date`"

	# have to bump RunNumber because cleanupOnExit decrements it 
	# once assumint completion is reason for stopping. On normal
	# completion RunNumber is one more than NumberOfCommands
	#
	# During abnormal completion, we need to emulate that behavior
	# before calling cleanupOnExit
	RunNumber=`/usr/bin/expr $RunNumber + 1`
	export DIAG_PKG_PASS=$RunNumber

	cleanupOnExit
	exit $CumulativeStatus
}

####################################
# check_command_status() function monitors the exit status of the
# executed command. It waits for exec_diag return and process the
# return status. Both the RunStatus and CumulativeStatus are updated 
# based on failing results.
#
#	check_command_status $command_name $finish_time $ALL(TRUE/FALSE)
#
# Messages are generated to output the result of the command.
####################################
check_command_status()
{
	# wait for current child to finish and get return status
	#
	export Child=$!
	wait $Child
	_testresult=$?
	export Child=0
	if [ $_testresult -eq 139 -o $_testresult -eq 237 ]
	then
		burninMessage "Command #$CmdNumber Run $DIAG_PKG_PASS Known issue ($_testresult) $_new_var"
		_testresult=16
	fi
	#
	# diagshow will only fail if a port has faulted.
	#
	_cmd=`commandPath exec_diag`
	$_cmd diagshow --slot $Slot -silent 1 -use_bports 1 >> $LogFile
	_showresult=$?
	if [ $_showresult -eq 139 -o $_showresult -eq 237 ]
	then
		burninMessage "Command #$CmdNumber Run $DIAG_PKG_PASS Known issue ($_showresult) diagshow"
		_showresult=0
	fi
	#  construct PASSED or FAILED msg from return value of command
	#  .  return value = 0 for PASSED
	#  .  return value != 0 for FAILED
	#
	# For all test status updating following table:
	#			        Normal test		"ALL" test
	#------------------------------------------------------------------
	#	Test P/F result		test return		"COMPLETE"
	#	Script cum status	test+show		show
	#	Blade LED		prev_cum+test+show	prev+show
	#
	# NOTE: that prev is handled in
	#  initialize_burnin_variables(). This reads the cumulative
	#  status from the non-volitile memory, and sets status and
	#  LED state based on that value. Therefore it does not need
	#  to be considered here.
	#
	# NOTE: REturn values of 237 or 139 idicate that there was a problem
	# with Linux actually executing the command. Since these commands never
	# were executed (diagshow or the actual command), their staus is ignored. 
	#
	if [ "$3" = "TRUE" -a $MultiBlade = TRUE ]
	then
		_stat=COMPLETE
		if [ $_showresult -ne 0 ]
		then
			CumulativeStatus=`/usr/bin/expr $CumulativeStatus + 1`
			RunStatus=1
			# --setled only takes effect if the command fails.
			# net result is turn on the LED since test
			# will always fail
			_cmd=`commandPath exec_diag`
			if [ $MultiBlade = FALSE ]
			then
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
			else
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
			fi
			burninMessage "Found error in $1 during run $RunNumber showresult $_showresult"
		fi
	else
		case "$_testresult" in
		0)
			_stat=PASSED;;
		16)
			_stat=SKIPPED;;
		*)
			_stat=FAILED;;
		esac
		if [ $_showresult -ne 0 -o "$_stat" = "FAILED" ]
		then
			CumulativeStatus=`/usr/bin/expr $CumulativeStatus + 1`
			RunStatus=1
			# --setled only takes effect if the command fails.
			# net result is turn on the LED since test
			# will always fail
			_cmd=`commandPath exec_diag`
			if [ $MultiBlade = FALSE ]
			then
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
			else
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
			fi
			burninMessage "Found error in $1 during run $RunNumber showresult $_showresult testresult $_testresult"
		fi
	fi
	burninMessage "Command #$CmdNumber Run $DIAG_PKG_PASS $_stat $1 `/bin/date`"
	_current_time=`/bin/date \+\%s`
	_duration=`/usr/bin/expr $_current_time - $2`
	_minute=`/usr/bin/expr $_duration / 60`
	_hour=`/usr/bin/expr $_minute / 60`
	_sec=`/usr/bin/expr $_duration % 60`
	_minute=`/usr/bin/expr $_minute % 60`
	burninMessage "Command #$CmdNumber Run $DIAG_PKG_PASS Duration: $1 took $_hour hr $_minute min $_sec sec ($_hour:$_minute:$_sec)"
}

##################################
# NAME
#    initialize_burnin_variables - Initializes the variables used
#                                  to control the burnin process
#
# SYNOPSIS
#    initialize_burnin_variables
#
# DESCRIPTION
#    This function initialize_burnin_variables() is used by scripts
#    that need values from non-volitile memory. For burnin, these
#    are RunNumber, and State of the overall burnin, and the
#    CumulativeStatus of the run. These are gathered from the serial 
#    EEPROM on each blade.
#
#    The number of runs to execute are contained in the PARAMDEF stored in 
#    config database.
#
# OPERANDS
#    NONE
#
##################################
initialize_burnin_variables()
{
	export RunNumber=`getBladeParam $Slot RUN_NUMBER`
	# Set for the diag to use in logging
	export DIAG_PKG_PASS=$RunNumber
	export State=ACTIVE
	CumulativeStatus=`getBladeParam $Slot CUM_STATUS`
	# Make sure LED's match test status after power cycle
	# exec_diag grabs all "--" arguments and executes them
	if [ $CumulativeStatus -ne 0 ]
	then
		# Failed already
		_cmd=`commandPath exec_diag`
		if [ $MultiBlade = FALSE ]
		then
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null &
		else
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null &
		fi
		burninMessage "Stored status indicates cumulative fail."
	else
		# Passing so far
		_cmd=`commandPath exec_diag`
		if [ $MultiBlade = FALSE ]
		then
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null &
		else
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null &
		fi
	fi
	# format up the start time for the logging messages
	_tmp=`/bin/cat /proc/dcomm`
	_sec=`/usr/bin/expr "$_tmp" : ".*time: \+\([0-9]\+\)"`
	_msec=`/usr/bin/expr "$_tmp" : ".*time: \+[0-9]\+:\([0-9]\+\)"`		  
	if { [ -z $_sec ] || [ -z $_msec ] ; } ; then
		_sec=0
		_msec=0
	fi
	export DIAG_PKG_BEG="$_sec:$_msec"

	# get the parameters for the script
	export RunStatus=`getBladeParam $Slot RUN_STATUS`
	export RunTime=`getBladeParam $Slot RUN_TIME`
	export TotalCmdNumber=`getBladeParam $Slot TOTAL_CMD_NUMBER`
	export CmdNumber=`getBladeParam $Slot CMD_NUMBER`
	export TotalThermal=`getBladeParam $Slot TEMP_CYCLE`
	export TotalVib=`getBladeParam $Slot VIB_CYCLE`
	export NumberOfRuns=`getConfig diag.$MyName.number_of_runs $INTEGER 1`
	export LogLength=`getConfig diag.$MyName.log_length $INTEGER 600000`
	if [ "$NumberOfRuns" = 1 ]
	then
	        ConfigSetCheck diag.$MyName.number_of_runs $INTEGER
		if [ $? != 0 ]
		then
		    setConfig diag.$MyName.number_of_runs $INTEGER 1
		fi
	fi
	Vib=`getConfig diag.$MyName.vib $INTEGER 0`
	if [ "$Vib" = 0 ]
	then
	        ConfigSetCheck diag.$MyName.vib $INTEGER
		if [ $? != 0 ]
		then
		    setConfig diag.$MyName.vib $INTEGER 0
		fi
	fi
	Thermal=`getConfig diag.$MyName.thermal $INTEGER 0`
	if [ "$Thermal" = 0 ]
	then
	        ConfigSetCheck diag.$MyName.thermal $INTEGER
		if [ $? != 0 ]
		then
		    setConfig diag.$MyName.thermal $INTEGER 0		  
		fi
	fi
}

##################################
# NAME
#    get_filename - Get the name of the current script
#
# SYNOPSIS
#    get_filename
#
# DESCRIPTION
#    This command reads the shell arguments and extract the name 
#    of the command referenced by the user. The preceding 
#    directories are stripped as are the trailing extentions.
#
#    The burnin log files are created at the same time. They are:
#    LogFile, and UserFile
#
# OPERANDS
#    NONE
#
##################################
get_filename()
{
	# Initialize incase of trap
	export StartTime=`/bin/date \+\%s`
	export Child=0
	RunNumber=0
	TotalCmdNumber=0
	TotalThermal=0
	TotalVib=0
	CmdNumber=0

	# setup some default values
	export CumulativeStatus=0
	export Slot=$_def_slot
	export DIAG_POST=1
	export DCOMM_USR_FAILLIM=16

	setdbg DIAG 1

	# get the main three: home, switch, and slot
	if [ ! -z "$FABOS_SLOTNO" ]
	then
		export Slot=$FABOS_SLOTNO
	else
		export FABOS_SLOTNO=$_def_slot
		export Slot=$FABOS_SLOTNO
	fi

	if [ -z "$FABOSHOME" ]
	then
		export FABOSHOME=/fabos
	fi

	if [ -z "$FABOS_SWITCHNO" ]
	then
		export FABOS_SWITCHNO=0
	fi

	export MultiBlade=`isMultiBlade`
	export Product=`getSWBD`
	if [ $MultiBlade = TRUE ]
	then
		slot_message=" $Slot"
	else
		slot_message=""
	fi
	# save my PID
	/bin/echo "$$ ACTIVE" > /tmp/burnin_PID.$FABOS_SWITCHNO.$Slot

	# setup the file names to use
	_tmp_dir=`getConfig diag.mode.burnin.logpath $STRING "/var/log"`
	if [ ! -d $_tmp_dir ]
	then
		/bin/echo "Could not find diag.mode.burnin.logpath $_tmp_dir using /var/log"
		_tmp_dir="/var/log"
	fi
	_log_path=`getConfig diag.$MyName.LogFilePath $STRING "$_tmp_dir"`
	if [ ! -d $_log_path ]
	then
		/bin/echo "Could not find diag.$MyName.LogFilePath $_log_path using $_tmp_dir"
		_log_path=$_tmp_dir
	fi
	export LogCmd=`commandPath logdata`
	export LogFile=$_log_path/$MyName.$Slot.log
	burninMessage "Sending logs to $LogFile"
	# walk to path to ensure path is available
	export UserFile="$MyName.$Slot.usr"
}

##################################
# NAME
#    get_command_list - Get the output of the create_command_file
#		        function to stdout
#
# SYNOPSIS
#    get_command_list
#
# DESCRIPTION
#    This command executes the create_command_file function and places
#    a "@" seperator after every line that is a command. Any line that
#    is not a command is eliminated from the output by the "isCommand"
#    filter.
#
#    No need to filter for comment lines or empty lines. The function 
#    isACommand will do that for us.
#
# OPERANDS
#    NONE
#
##################################
get_command_list()
{
	create_command_file | while read _line
	do
		if [ `isACommand "$_line"` = "command" ]
		then
			/bin/echo $_line@

		fi
	done
}

##################################
# NAME
#    get_config_list - Get the output of the configShow diag entries
#		       to stdout.
#
# SYNOPSIS
#    get_config_list
#
# DESCRIPTION
#    This command executes the configShow function and places
#    a "@" seperator after every input line that belongs to the 
#    calling script. Any input line that is not related to the calling
#    script is eliminated from the output by the "/bin/grep diag.$MyName"
#    filter.
#
# OPERANDS
#    NONE
#
##################################
get_config_list()
{
	_cmd=`commandPath configShow`
	$_cmd  2>/dev/null | /bin/grep diag.$MyName | while read _line
	do
		/bin/echo $_line@
	done
}

##################################
# NAME
#    get_config_entry - Get the value of a "configVariable:value" pair
#		        from the output generated by get_config_list.
#
# SYNOPSIS
#    get_config_entry
#
# DESCRIPTION
#    This command searches the output of config_list until a match on
#    the traget is found. The sed outputs the string starting from the
#    match of $target until the "@" seperator. The "cut" on ":" delimiter
#    trims off $target and the ":". The "cut" on "@" delimiter
#    removes the "@" sight leaving the value of the config variable
#    named $target.
#
# OPERANDS
#    $1 = target to match in the string
#    config_list = variable containing the string to search
#
##################################
get_config_entry()
{
	target=$1
	if [ ! -z "`/bin/echo $config_list | /bin/grep $target`" ]
	then
		/bin/echo $config_list | \
		     /bin/sed -e "s/^.*\(${target}:[[:print:]]\+\).*@/\1/g" |\
		     /usr/bin/cut -d":" -f2 | \
		     /usr/bin/cut -d"@" -f1
	fi
}

##################################
# NAME
#    get_param_list - Get the output of the create_param_file
#		        function to stdout
#
# SYNOPSIS
#    get_param_list
#
# DESCRIPTION
#    This command executes the create_param_file function and places
#    a "@" seperator after every line that is a paramdef. Any line that
#    is not a param is eliminated from the output by the "PARAMDEF"
#    filter.
#
#    Added filter for lines that are comments. They could be commented
#    out PARAMDEFs.
#
# OPERANDS
#    NONE
#
##################################
get_param_list()
{
	create_param_file | /bin/grep -v "^#" | while read _line
	do
		if [ ! -z "`/bin/echo $_line | /bin/grep 'PARAMDEF'`" ]
		then
			/bin/echo $_line@

		fi
	done
}

##################################
# NAME
#    get_match_list - Get a list of the lines that contain the string
#		      $2 to stdout
#
# SYNOPSIS
#    get_match_list <file> <match_string>
#
# DESCRIPTION
#    This searches file named $1 for string $2 and places a @seperated
#    list of the lines that matches were found.
#
#    The output is a single line with "@" seperated
#    match strings.
#
# OPERANDS
#    NONE
#
##################################
get_match_list()
{
	_fil=$1
	shift
	/bin/cat "$_fil" | /bin/grep -n $* | while read _line
	do
		/bin/echo $_line@
	done
}

##################################
# NAME
#    get_error_from_file - Get the error entries in a log file
#
# SYNOPSIS
#    get_error_from_file <file>
#
# DESCRIPTION
#    This searches file named $1 for string ERROR and places a @seperated
#    list of the lines that matches were found.
#
# OPERANDS
#    NONE
#
##################################
get_error_from_file()
{
	_list=`get_match_list $1 "-e ERROR -e BLOOM -e BLADE"`
	_count=1
	_lin=`/bin/echo $_list | /usr/bin/cut -d"@" -f$_count`
	while [ "$_lin" != "" ]
	do
		# get the line number of the ERROR, BLOOM or BLADE
		num1=`/bin/echo $_lin | /usr/bin/cut -d":" -f1`
		# get the actual line
		newlin=`/bin/cat $1 | /bin/grep -n "^" | /bin/grep -e "^${num1}:" | /usr/bin/cut -d":" -f2-`
		if [ "`/bin/echo $newlin | /bin/grep -e BLOOM -e BLADE`" != "" ]
		then
			# BLOOM log processing
			/bin/echo $newlin
		else
			done=`/bin/echo $newlin | /bin/grep "Err#"`
			while [ "$done" = "" ]
			do
				/bin/cat $1 | /bin/grep -n "^" | \
				/bin/grep -e "^${num1}:" | /usr/bin/cut -d":" -f2-
				num1=`/usr/bin/expr $num1 + 1`
				newlin=`/bin/cat $1 | /bin/grep -n "^" | /bin/grep -e "^${num1}:" | /usr/bin/cut -d":" -f2-`
				done=`/bin/echo $newlin | /bin/grep "Err#"`
			done
			/bin/cat $1 | /bin/grep -n "^" | \
			/bin/grep -e "^${num1}:" | /usr/bin/cut -d":" -f2-
		fi
		_count=`/usr/bin/expr $_count + 1`
		_lin=`/bin/echo $_list | /usr/bin/cut -d"@" -f$_count`
	done
}

##################################
# NAME
#    get_driver_errors - Searches the burninerr log for bloom driver
#			 or blade driver errors during the current 
#			 burnin run.
#
# SYNOPSIS
#    get_driver_errors
#
# DESCRIPTION
#    This function get_driver_errors() assumes that the entries
#    in the burninerrshow output are in cronological order.
#    It also assumes that a burnin cycle is active, and that it must
#    have a start message before this function has been called.
#
#    The entire burnin error log is sampled to determine if there are
#    any BLADE/BLOOM entries at all. If some are detected, then the
#    last burnin start message is located, and its line number in the
#    output is saved. If a start mesage is not found, then it is
#    assumed that the errors are valid, and a message is output, and
#    fail status returned.
#
#    Once the position of the last start message is located, then the
#    position of each BLOOM error is compared to that of the start. If
#    the error appears after the start, then the error is considered
#    within this run, and error messages and status are sent. The same
#    process is performed for the BLADE errors.
#
# RESULT
#    The result of the testing is found by the existance of the file
#    /var/log/driver_stat_$Slot. If the file exists, then an error was
#    found. If it does not exist, then no error was found.
#
#    It is the callers responsibility to remove the file after testing
#    for the result. This function will remove it prior to the evaluation
#    to ensure the accuracy of the testing.
#
# OPERANDS
#    $1 after_inc - inicates the function was called after the
#	RunNumber was incremented at the end of the run.
#
##################################
get_driver_errors()
{
	if [ "$1" == "after_inc" ]
	then
		_run=`/usr/bin/expr $RunNumber - 1`
	fi
	_cmd=`commandPath burninerrshow`
	/bin/rm -f  /tmp/driver_stat_$Slot
	$_cmd $Slot > /tmp/$$.tmp
	if [ "`/bin/grep -e BLOOM -e BLADE /tmp/$$.tmp | /bin/grep -v Warn`" != "" ]
	then
		_last_start=`cat /tmp/$$.tmp | \
				/usr/bin/tr -s "\000 " " " | \
				/bin/grep -n Starting | \
				/usr/bin/tail -n1 | \
				/usr/bin/cut -d":" -f1`
		if [ "$_last_start" != "" ]
		then
			cat /tmp/$$.tmp | \
			/usr/bin/tr -s "\000 " " " | \
			/bin/grep -n BLOOM | \
			/bin/grep -v Warn |\
			/usr/bin/cut -d":" -f1 | \
			while read bloom_line
			do
				if [ ! -z "$bloom_line" ]
				then
					if [ $bloom_line -gt $_last_start ]
					then
						touch /tmp/driver_stat_$Slot
						burninMessage "BLOOM errors detected during run $_run"
						break;
					fi
				fi
			done
			if [ ! -f /tmp/driver_stat_$Slot ]
			then
				cat /tmp/$$.tmp | \
				/usr/bin/tr -s "\000 " " " | \
				/bin/grep -n BLADE | \
				/bin/grep -v Warn |\
				/usr/bin/cut -d":" -f1 | \
				while read blade_line
				do
					if [ ! -z "$blade_line" ]
					then
						if [ $blade_line -gt $_last_start ]
						then
							touch /tmp/driver_stat_$Slot
							burninMessage "BLADE errors detected during run $_run"
							break;
						fi
					fi
				done
			fi
		else
			touch /tmp/driver_stat_$Slot
			/burninMessage "BLOOM/BLADE errors detected during run $_run, but no Start message"
		fi
	fi
	/bin/rm -f  /tmp/$$.tmp
}

##################################
# NAME
#    get_any_errors - Searches the burninerr log for any errors during
#		      the current burnin run.
#
# SYNOPSIS
#    get_any_errors
#
# DESCRIPTION
#    This function get_any_errors() assumes that the entries
#    in the burninerrshow output are in cronological order.
#    It also assumes that a burnin cycle is active, and that it must
#    have a start message before this function has been called.
#
#    The entire burnin error log is sampled to determine if there are
#    any error entries at all. If some are detected, then the
#    last burnin start message is located, and its line number in the
#    output is saved. If a start mesage is not found, then it is
#    assumed that the errors are valid, and a message is output, and
#    fail status returned.
#
#    Once the position of the last start message is located, then the
#    position of each error is compared to that of the start. If
#    the error appears after the start, then the error is considered
#    within this run, and error messages and status are sent.
#
# RESULT
#    The result of the testing is found by the existance of the file
#    /var/log/error_stat_$Slot. If the file exists, then an error was
#    found. If it does not exist, then no error was found.
#
#    It is the callers responsibility to remove the file after testing
#    for the result. This function will remove it prior to the evaluation
#    to ensure the accuracy of the testing.
#
# OPERANDS
#    NONE
#
##################################
get_any_errors()
{
	_cmd=`commandPath burninerrshow`
	/bin/rm -f  /tmp/error_stat_$Slot
	$_cmd $Slot > /tmp/$$.tmp
	if [ "`/bin/grep -e "^Switch: " /tmp/$$.tmp | /bin/grep -v Warn`" != "" ]
	then
		_last_start=`cat /tmp/$$.tmp | \
				/usr/bin/tr -s "\000 " " " | \
				/bin/grep -n Starting | \
				/usr/bin/tail -n1 | \
				/usr/bin/cut -d":" -f1`
		if [ "$_last_start" != "" ]
		then
			cat /tmp/$$.tmp | \
			/usr/bin/tr -s "\000 " " " | \
			/bin/grep -n "^Switch:" | \
			/bin/grep -v Warn |\
			/bin/grep -v "WARNING: High temp" |\
			/usr/bin/cut -d":" -f1 | \
			while read error_line
			do
				if [ ! -z "$error_line" ]
				then
					if [ $error_line -gt $_last_start ]
					then
						touch /tmp/error_stat_$Slot
						burninMessage "Errors detected during run $_run"
						break;
					fi
				fi
			done
		else
			touch /tmp/error_stat_$Slot
			burninMessage "Errors detected during run $_run, but no Start message"
		fi
	fi
	/bin/rm -f  /tmp/$$.tmp
}

##################################
# NAME
#    setup_environment - Sets up the working environment of script
#
# SYNOPSIS
#    setup_environment
#
# DESCRIPTION
#    This function setup_environment() initialized the shell
#    environment with the necessary environment variables
#    to function on a blade within the fabos environment.
#    If any of the mandatory variables are not available,
#    the script will exit with a value of 3.
#
#    The action of modifying, or ensuring a value for each 
#    variable is called based on the input parameters to 
#    the script (diagsetcycle). If diagsetcycle is specified, 
#    then the script prompts the user and exits successfully.
#
#    If diagsetcycle is not specified, then the paramdef 
#    files are processed nto the config database, and exported 
#    into the shell environment.
#
# OPERANDS
#    NONE
#
##################################
setup_environment()
{
	#
	# Begin processing the PARAMETERS
	#
	import_burnin_variables

	#
	# Begin processing the COMMAND file content
	#
	export CommandList=`get_command_list`
	export NumberOfCommands=`/bin/echo $CommandList | \
				 /usr/bin/tr -s " " "_" | \
				 /usr/bin/tr -s "@" " " | \
				 /usr/bin/wc -w`
}
##################################
# NAME
#    truncate_log - Truncate log to prevent file system overflow
#
# SYNOPSIS
#    truncate_log
#
# DESCRIPTION
#    This function truncate_log() determine if the log file is greater
#    than $LogLength characters in size, and truncate  it to 3/4 the
#    number of lines after putting all errors in the file at the
#    top of the file.
#
#    NOTE: that if the LogLength is too small and/or the number of
#    errors is great, that the resulting file can be larger than LogLength.
#
# check log file length, trim the log file if necessary
# Trim log file if greater than LogLength (300000 default) characters
# 300000 X 8 blades = 2.4 meg * 2 = 4.8 Meg (*2 max growth per test)
# tmp logs 3/4 4.8 meg = 3.6 Meg + 4.8 Meg = 7.4 Meg of log files
# 8 Meg is about all the space we have to operate in.
#
# OPERANDS
#    NONE
#
##################################
truncate_log()
{
	if [ $LogLength -ne 0 ]
	then
		_length_of_log=`/usr/bin/wc $LogFile | \
			/usr/bin/tr -s " " " " | \
			/usr/bin/cut -d" " -f4`
		if [ $_length_of_log -gt $LogLength ]
		then
			# get all the errors from the current file
			get_error_from_file $LogFile > $LogFile.err
			# trim the file by 3/4 lines
			_length_of_log=`/usr/bin/wc -l $LogFile | \
				/usr/bin/tr -s " " " " | \
				/usr/bin/cut -d" " -f2`
			_new_length=`/usr/bin/expr $_length_of_log / 4`
			_new_length=`/usr/bin/expr $_new_length \* 3`
			/usr/bin/tail -n $_new_length $LogFile > $LogFile.tmp
			/bin/cat $LogFile.err $LogFile.tmp > $LogFile
			/bin/rm -f $LogFile.tmp $LogFile.err
			burninMessage "Trimmed log from $_length_of_log to $_new_length"
			/bin/sync
		fi
	fi
}
##################################
#PUBLIC FUNCTIONS
##################################
##################################
# This function burninMessage() is intended for
# internal use only. It formats the output messages during burnin to
# be blade product sensative.
#
# It gets the message to output as the input parameters, and prepends
# slot information if appropriate.
# $* = message to output
##################################
burninMessage()
{
	/bin/echo "$slot_message $MyName: $*"
	$LogCmd $LogFile "$slot_message $MyName: $*"
}
##################################
# NAME
#    getSWBD - Output the software board ID
#
# SYNOPSIS
#    getSWBD
#
# DESCRIPTION
#    This function parses the output of the sin command to extract the
#    SWBD value for this platform.
#
# OPERANDS
#    NONE
##################################
getSWBD()
{
	_abc=`/sbin/sin | \
		  /bin/grep Platform | \
		  /usr/bin/cut -d" " -f2 | \
		  /usr/bin/cut -d"," -f1`
	echo $_abc
}
##################################
# NAME
#    isMultiBlade - Output TRUE if product has multiple blades
#
# SYNOPSIS
#    isMultiBlade
#
# DESCRIPTION
#    This function parses the output of the sin command to extract the
#    SWBD value for this platform, then it uses that information to
#    determine a TRUE/FALSE answer.
#
# OPERANDS
#    NONE
##################################
isMultiBlade()
{
	_abc=`/sbin/sin | \
		  /bin/grep Platform | \
		  /usr/bin/cut -d" " -f2 | \
		  /usr/bin/cut -d"," -f1`
	case $_abc in 
	"SWBD10" ) /bin/echo "TRUE";;
	* ) /bin/echo "FALSE";;
	esac
}
##################################
# NAME
#    getAvailSlots - Output a space seperated list of the slots on this switch
#
# SYNOPSIS
#    getAvailSlots
#
# DESCRIPTION
#    This function parses the output of the diagshow command to extract the
#    list of slots available in this switch.
#
# OPERANDS
#    NONE
##################################
getAvailSlots()
{
	_cmd=`commandPath exec_diag`
	_abc=`$_cmd diagshow | \
		/bin/grep Slot | \
		/usr/bin/cut -d " " -f2 | \
		/usr/bin/xargs /bin/echo`
	echo "$_abc"
}
##################################
# NAME
#    isValidSlot - Output TRUE if valid slot number otherwise FALSE
#
# SYNOPSIS
#    isValidSlot
#
# DESCRIPTION
#    This function validate the input user slot number if
#    it is a valid or not by comparing with the result of getAvailSlots.
#    Then it will return TRUE/FALSE answer
# OPERANDS
#     $1 = user input slot number for validation
##################################
isValidSlot()
{

slot="$1"
list=`getAvailSlots`
if [ -z $1 ] ; then
   /bin/echo "FALSE"
else
     if [ "`/bin/echo $list | /bin/grep -w $slot`" = "" ] ; then
	/bin/echo "FALSE"
     else
	/bin/echo "TRUE"
     fi
fi

}
##################################
# NAME
#    checkBurninState - Determines if burnin can continue
#
# SYNOPSIS
#    checkBurninState
#
# DESCRIPTION
#    This function checkBurninState() handles getting the script
#    variables initialized, and handles the diagsetcycle functionlity.
#
#    If the scripts is not in diagsetcycle mode, then it
#    determines if it is OK to start a burnin run. 
#    If the burnin State is COMPLETE, then it is not safe to start 
#    burnin. This differentiates a real start from a power cycle of 
#    the same burnin run.
#
# OPERANDS
#    NONE
##################################
checkBurninState()
{
	# get my name
	export debug_on=0
	_num_sub=`/bin/echo $0 | /usr/bin/tr "\057" " " | /usr/bin/wc -w`
	_num_sub=`/usr/bin/expr $_num_sub + 1`
	export MyName=`/bin/echo $0 | /usr/bin/tr "\057" "@" | \
			  /usr/bin/cut -d"@" -f$_num_sub | \
			  /usr/bin/cut -d"." -f1`
	if [ -z "$MyName" ]
        then
		_num_sub=`/bin/echo $0 | /usr/bin/tr "\057" " " | /usr/bin/wc -w`
		export MyName=`/bin/echo $0 | /usr/bin/tr "\057" "@" | \
				  /usr/bin/cut -d"@" -f$_num_sub | \
				  /usr/bin/cut -d"." -f1`
	fi
	export FileName=`/bin/echo $0 | /usr/bin/tr "\057" "@" | \
			    /usr/bin/cut -d"@" -f$_num_sub`


	# if only diagsetcycle, then process the environment
	if [ "$1" = "diagsetcycle" ]
	then
		export State=DIAGSETCYCLE
		process_diagsetcycle $*
		exit 0
	fi

	get_filename
	_stored_state=`getBladeParam $Slot STATE`
	CumulativeStatus=`getBladeParam $Slot CUM_STATUS`

	if [ "$_stored_state" = "COMPLETED" -o "$_stored_state" = "ABORT"  ] 
	then
		/bin/echo "$$ COMPLETE" > /tmp/burnin_PID.$FABOS_SWITCHNO.$Slot
		/bin/echo "The last $MyName run ran to completion."
		/bin/echo "Please run diagSetBurnin if you wish to begin a new run."
		/bin/echo "    or run diagStopBurnin if you wish to exit burnin mode."
		_cmd=`commandPath exec_diag`
		if [ $CumulativeStatus = 0 ]
		then
			# We have passed
			if [ $MultiBlade = FALSE ]
			then
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_FAST_BLINK > /dev/null 2>/dev/null &
				_cmd=`commandPath switchbeacon`
				$_cmd 1
			else
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_DEFAULT > /dev/null 2>/dev/null &
				_cmd=`commandPath bladebeacon`
				$_cmd $Slot 1
			fi
		else
			# We have failed
			if [ $MultiBlade = FALSE ]
			then
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
				_cmd=`commandPath switchbeacon`
				$_cmd 1
			else
				eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
				_cmd=`commandPath bladebeacon`
				$_cmd $Slot 1
			fi
		fi
		exit $CumulativeStatus
	else
		_script_name=`getBladeParam $Slot SCRIPT_NAME`
		_num_sub=`/bin/echo $_script_name | /usr/bin/tr "\057" " " | /usr/bin/wc -w`
		_num_sub=`/usr/bin/expr $_num_sub + 1`
		_script_name=`/bin/echo $_script_name | /usr/bin/tr "\057" "@" | \
					/usr/bin/cut -d"@" -f$_num_sub`
		if [ -z "$_script_name" ]
		then
			_num_sub=`/bin/echo $0 | /usr/bin/tr "\057" " " | /usr/bin/wc -w`
				_script_name=`/bin/echo $_script_name | /usr/bin/tr "\057" "@" | \
				  /usr/bin/cut -d"@" -f$_num_sub`
		fi
		if [ "$_script_name" != "$FileName" ]
		then
			/bin/echo "$$ COMPLETE" > /tmp/burnin_PID.$FABOS_SWITCHNO.$Slot
			burninMessage "Found wrong script specified, Please run diagSetBurnin"
			burninMessage "I am: $FileName, database is: $_script_name"
			_cmd=`commandPath exec_diag`
			if [ $CumulativeStatus = 0 ]
			then
				# We have passed
				if [ $MultiBlade = FALSE ]
				then
					eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_FAST_BLINK > /dev/null 2>/dev/null &
					_cmd=`commandPath switchbeacon`
					$_cmd 1
				else
					eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_DEFAULT > /dev/null 2>/dev/null &
					_cmd=`commandPath bladebeacon`
					$_cmd $Slot 1
				fi
			else
				# We have failed
				if [ $MultiBlade = FALSE ]
				then
					eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
					_cmd=`commandPath switchbeacon`
					$_cmd 1
				else
					eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
					_cmd=`commandPath bladebeacon`
					$_cmd $Slot 1
				fi
			fi
			exit $CumulativeStatus
		fi


		setup_environment $*

		# Save the fact that we went live
		export State=ACTIVE

		#
		# Get the cp slot, and post to CP log
		#
		_cmd=`commandPath getCPSlot`
		_cp_slot=`$_cmd | /usr/bin/cut -d" " -f3`
		_cmd=`commandPath exec_diag`

		_first_slot=`$_cmd diagshow | \
				/bin/grep Slot | \
				/usr/bin/cut -d " " -f2 | \
				/usr/bin/xargs echo | \
				/usr/bin/cut -d " " -f1`

	        if [ "$FABOS_SWITCHNO" -ne 0 ]
		then
		    export FABOS_SWITCHNO=0
		    _sw0_first_slot=`$_cmd diagshow | \
				/bin/grep Slot | \
				/usr/bin/cut -d " " -f2 | \
				/usr/bin/xargs echo | \
				/usr/bin/cut -d " " -f1`
		    export FABOS_SWITCHNO=1

                    if [ ! -z "$_sw0_first_slot" ]
		    then
			 _first_slot=$_sw0_first_slot
		    fi
		fi

		_date=`/bin/date`
		_cmd=`commandPath version`
		_version=`$_cmd | /bin/grep Fabric | /usr/bin/tr -s " " " " | /usr/bin/cut -d " " -f3`
		if [ $MultiBlade = TRUE ]
		then
			if [ $_first_slot -eq $Slot ]
			then
				_message=" $Slot CP $_cp_slot Starting $MyName run $RunNumber $_version $_date"
				# Remove double spaces in the message for the forceError command
				_message=`/bin/echo "$_message" | /usr/bin/tr -s " " " "`
				_cmd=`commandPath exec_diag`
				$_cmd forceError --slot $_cp_slot -severity 4 -message '"'"$_message"'"'
			fi
	        fi

		# get the run and command numbers
		initialize_burnin_variables
		if [ "$_stored_state" = "INITIALIZED" ]
		then
			burninMessage "Run diagClearError"
			_cmd=`commandPath exec_diag`
			$_cmd diagClearError $Slot

			export StartTime=`/bin/date \+\%s`
		else
			export StartTime=`getBladeParam $Slot START_TIME`
			# Check burninerrlog to see if there were any fails since we
			# have started. There could be fails on the test that was
			# interupted during a power cycle. We want to tag a blade as
			# fail even if the test was interupted.
			get_any_errors
			if [ -f /tmp/error_stat_$Slot ]
			then
				rm -f /tmp/error_stat_$Slot
				CumulativeStatus=`expr $CumulativeStatus + 1`
			fi
			if [ "`/sbin/bootenv | /bin/grep BadRootDev`" != "" ]
			then
				$LogCmd $LogFile "Found BadRootDev in bootenv"
			fi		
		fi

		#
		# Get the user port list. This MUST be after the
		# diagClearError on the first run.
		#
		_cmd=`commandPath exec_diag`
		export UserPorts=`$_cmd diagshow -use_bports 1 | \
			 /bin/grep "   OK" | \
			 /usr/bin/cut -d " " -f 2 | \
			 /bin/grep -v "\-\-" | /bin/sort -n | \
			 /usr/bin/xargs /bin/echo`

		# If we have not executed a command yet, then post
		# start message
		if [ $RunNumber -eq 1 -a $CmdNumber -eq 1 ]
		then
			export CmdNumber=`expr $NumberOfCommands + 1`
			_message="Starting run $_version $_date"
			# Remove double spaces in the message for the forceError command
			_message=`/bin/echo "$_message" | /usr/bin/tr -s " " " "`
			_cmd=`commandPath exec_diag`
			if [ $MultiBlade = TRUE ]
			then
				$_cmd forceError --slot $Slot -severity 4 -message '"'" $Slot $MyName: $_message"'"'
			else
				$_cmd forceError --slot $Slot -severity 4 -message '"'" $MyName: $_message"'"'
			fi
			burninMessage "Starting run `/bin/date`"
			if [ "$label" != "-" ]
			then
				burninMessage "Label $label"
			fi
		fi
		# ensure log mode is correct for both the CP and blade
		_cmd=`commandPath burninLogMode`
		if [ $MultiBlade = TRUE ]
		then
			$_cmd -slot $_cp_slot -mode FIRST_IN
		fi
		$_cmd -slot $Slot -mode FIRST_IN
		saveBladeParams $Slot

		# get setup for quick exit
		trap diagcommon_cleanup INT TERM
	fi
}

##################################
# NAME
#    initializePerRunVariables - initializes the variables
#                                active per burnin run
#
# SYNOPSIS
#    initializePerRunVariables
#
# DESCRIPTION
#    Initializes the variables used to control the per run loop
#    activity, bump run counter, initialize the command counter,
#    and run status to pass.
#
# OPERANDS
#    NONE
##################################
initializePerRunVariables()
{
	if [ $CmdNumber -gt $NumberOfCommands ]
	then
		export RunStatus=0
		export CmdNumber=1
		export RunTime=`/bin/date \+\%s`
		saveBladeParams $Slot
	fi
	burninMessage "Begin run $RunNumber of $NumberOfRuns at command $CmdNumber `/bin/date`"
}

##################################
# NAME
#    isACommand - Sorts the command lines from a command entry
#
# SYNOPSIS
#    isACommand
#
# DESCRIPTION
#    This function isACommand() determines if the line in the 
#    $CommandList is an executable, or a comment or other entry. 
#    It assumes that blank lines and lines that start with "#" are 
#    comments or non-executible entries. All other lines are 
#    considered a command.
#
# OPERANDS
#    NONE
##################################
isACommand()
{
	if [ "$*" -a -z "`/bin/echo $* | /bin/grep '#'`" ]
	then
		/bin/echo command
	else
		/bin/echo "other"
	fi
}

##################################
# NAME
#    runBurninCommand - executes the commands from a $CommandList variable
#
# SYNOPSIS
#    runBurninCommand enable command argument ...
#
# DESCRIPTION
# This function runBurninCommand() processes the argument string as a command,
# it updates the command number, and updates the status.
#
# OPERANDS
#  $1 enable  - If the command is to be run
#  $2 command - The command name to run
#  $*         - The command name to run with all its arguments
#
##################################
runBurninCommand()
{
	# get the command from the list, and get the relavent parts
	_command=`/bin/echo $CommandList | /usr/bin/cut -d"@" -f$CmdNumber`
	_command_name=`/bin/echo $_command | /usr/bin/cut -d" " -f2`
	_command_args=`/bin/echo $_command | /usr/bin/cut -d" " -f3-`
 	_ports=`/bin/echo $_command_args | /bin/grep -w "UserPorts"`
	_ALL=FALSE
	if { [ "$_command_name" = "spinsilk" ] \
	    || [ "$_command_name" = "backport" ] \
	    || [ "$_command_name" = "spinSilk" ] \
	    || [ "$_command_name" = "backPort" ] ; }
	then
		if [ "$_ports" = "" ]
		then
		    _ALL=TRUE
		fi
	fi
	_cmd_time=`/bin/date \+\%s`
	_var=`eval /bin/echo $_command_name $_command_args`
	_new_var=`eval /bin/echo $_var`
	_enable=`/bin/echo $_command | /usr/bin/cut -d" " -f1`
	_enable=`eval /bin/echo $_enable`
	_val_enable=`eval /bin/echo $_enable`
	if [ "$_val_enable" = "0" ]
	then
		burninMessage "Command #$CmdNumber Run $DIAG_PKG_PASS - Skip $_new_var"
	else
		shift
		burninMessage "Command #$CmdNumber Run $DIAG_PKG_PASS - Run $_new_var"
		$_new_var 2>&1 >> $LogFile &
		check_command_status $_command_name $_cmd_time $_ALL
	fi

	# Increment the number of completed commands
	# command index is already CmdNumber + 1
	CmdNumber=`/usr/bin/expr $CmdNumber + 1`
	TotalCmdNumber=`/usr/bin/expr $TotalCmdNumber + 1`
	if [ $CmdNumber -gt $NumberOfCommands ]
	then
		RunNumber=`/usr/bin/expr $RunNumber + 1`
		# Set for the diag to use in logging
		export DIAG_PKG_PASS=$RunNumber
	fi
	truncate_log

	# Flush the data to flash
	/bin/sync

	saveBladeParams $Slot
}

##################################
# NAME
#    burninStatusUpdate - Updates the global variables per burnin run
#
# SYNOPSIS
#    burninStatusUpdate
#
# DESCRIPTION
#    This function burninStateUpdate()handles the status updating 
#    for a burnin script it modifies the CumulativeStatus based
#    on the RunStatus stored in the serial EEPROM of the blade.
#
#    Closing messages are output if this was the last run of burnin, 
#    and the results for this run are stored in the serial EEPROM
#
# OPERANDS
#    NONE
##################################
burninStatusUpdate()
{
	runnum=`getBladeParam $Slot RUN_NUMBER`
	runnum=`/usr/bin/expr $runnum - 1`
	RunStatus=`getBladeParam $Slot RUN_STATUS`
	TotalCmdNumber=`getBladeParam $Slot TOTAL_CMD_NUMBER`
	_current_time=`/bin/date \+\%s`
	_duration=`/usr/bin/expr $_current_time - $RunTime`
	_minute=`/usr/bin/expr $_duration / 60`
	_hour=`/usr/bin/expr $_minute / 60`
	_sec=`/usr/bin/expr $_duration % 60`
	_minute=`/usr/bin/expr $_minute % 60`

	TotalThermal=`/usr/bin/expr $TotalThermal + $Thermal`
	TotalVib=`/usr/bin/expr $TotalVib + $Vib`

	# get existance of any driver errors
	_driver_stat=0
	get_driver_errors before_inc
	if [ -f /tmp/driver_stat_$Slot ]
	then
		rm -f /tmp/driver_stat_$Slot
		_driver_stat=1
	fi
	if [ $RunStatus -eq 0 -a $_driver_stat -eq 0 ]
	then
		_message=PASSED
	else
		_message=FAILED
		CumulativeStatus=`expr $CumulativeStatus + 1`
		if [ $MultiBlade = FALSE ]
		then
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
		else
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
		fi
		burninMessage "Found error during run $runnum RunStatus $RunStatus driver_stat $_driver_stat"
	fi
	burninMessage "END RUN $runnum of $NumberOfRuns $_message `/bin/date` Thermal $TotalThermal Vib $TotalVib took $_hour hr $_minute min $_sec sec ($_hour:$_minute:$_sec)"
	#
	# done after message so previous message will be correct
	#
	/bin/sync
}

##################################
# NAME
#    cleanupOnExit - Cleans all temporary files at termination of script
#
# SYNOPSIS
#    cleanupOnExit
#
# DESCRIPTION
#    Removes any working files created by the script, and calls cleanup
#    routine in the main script (my_local_cleanup())
# 
# OPERANDS
#    NONE
##################################
cleanupOnExit()
{
	#
	# Save time data for ending summary
	#
	_run_number=`/usr/bin/expr $RunNumber - 1`
	export DIAG_PKG_PASS=$_run_number
	_current_time=`/bin/date \+\%s`
	_duration=`/usr/bin/expr $_current_time - $StartTime`
	_minute=`/usr/bin/expr $_duration / 60`
	_hour=`/usr/bin/expr $_minute / 60`
	_sec=`/usr/bin/expr $_duration % 60`
	_minute=`/usr/bin/expr $_minute % 60`

	#
	# wait for all active burnins to complete to set final status
	#
	_some_active=TRUE
	/bin/echo "$$ COMPLETE" > /tmp/burnin_PID.$FABOS_SWITCHNO.$Slot
	#
	# When all are done, determine final status
	# Get status before we start to wait for the other blades to complete
	#
	_cmd=`commandPath exec_diag`
	$_cmd diagshow --slot $Slot -silent 1 -use_bports 1  >> $LogFile
	_showresult=$?
	if [ $_showresult = 237 -o $_showresult = 139 ]
	then
		burninMessage "Known issue diagshow after run complete when cleaning up. showresult $_showresult"
		_showresult=0
	fi
	# get existance of any driver errors
	_driver_stat=0
	get_driver_errors after_inc
	if [ -f /tmp/driver_stat_$Slot ]
	then
		rm -f /tmp/driver_stat_$Slot
		_driver_stat=1
	fi
	_showresult=`/usr/bin/expr $_showresult + $_driver_stat`
	# If we have failed (ever), lite the light and
	# adjust status
	if [ $_showresult -ne 0 ]
	then
		CumulativeStatus=`/usr/bin/expr $CumulativeStatus + 1`
		# --setled only takes effect if the command fails.
		# net result is turn on the LED since test
		# will always fail
		_cmd=`commandPath exec_diag`
		if [ $MultiBlade = FALSE ]
		then
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
		else
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
		fi
		burninMessage "Found error after run complete when cleaning up. showresult $_showresult"
	fi
	while [ $_some_active = TRUE ]
	do
		_some_active=FALSE
		for fil in `ls /tmp/burnin_PID.$FABOS_SWITCHNO.*`
		do
			_state=`/bin/cat $fil | /usr/bin/cut -d" " -f2`
			if [ "$_state" = ACTIVE ]
			then
				_some_active=TRUE
			fi
		done
		if [ "$_some_active" = "TRUE" ]
		then
			sleep 1
		else
			#
			# When all are done, determine final status
			# Get status one last time before exiting to
			# allow any ALL test to complete
			#
			_cmd=`commandPath exec_diag`
			$_cmd diagshow --slot $Slot -silent 1 -use_bports 1 >> $LogFile
			_showresult=$?
			if [ $_showresult = 237 -o $_showresult = 139 ]
			then
				burninMessage "Known issue diagshow after all blades complete their run when cleaning up. showresult $_showresult"
				_showresult=0
			fi
			# get existance of any driver errors
			_driver_stat=0
			get_driver_errors after_inc
			if [ -f /tmp/driver_stat_$Slot ]
			then
				rm -f /tmp/driver_stat_$Slot
				_driver_stat=1
			fi
			_showresult=`/usr/bin/expr $_showresult + $_driver_stat`
			# If we have failed (ever), lite the light and
			# adjust status
			if [ $_showresult -ne 0 ]
			then
				CumulativeStatus=`/usr/bin/expr $CumulativeStatus + 1`
				# --setled only takes effect if the command fails.
				# net result is turn on the LED since test
				# will always fail
				_cmd=`commandPath exec_diag`
				if [ $MultiBlade = FALSE ]
				then
					eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SLOW_BLINK > /dev/null 2>/dev/null
				else
					eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_SOLID_ON > /dev/null 2>/dev/null
				fi
				burninMessage "Found error after all blades complete their run when cleaning up. showresult $_showresult"
			fi
		fi
	done

	#
	# Save final data to blade storage
	#
	if [ "$State" != "DIAGSETCYCLE" ]
	then
		saveBladeParams $Slot
	fi
	#
	# Set exit status now that we are done
	#
	if [ $CumulativeStatus -eq 0 ]
	then
		_cum_state=PASSED
	else
		_cum_state=FAILED
	fi

	#
	# Output a ending summary
	#
	_cmd=`commandPath version`
	_version=`$_cmd | /bin/grep Fabric | /usr/bin/tr -s " " " " | /usr/bin/cut -d " " -f3`
	_message="$State stat $_cum_state $TotalCmdNumber cmds in $_run_number runs $_version Therm $TotalThermal Vib $TotalVib in $_hour hr $_minute min $_sec sec ($_hour:$_minute:$_sec)"
	# Remove double spaces in the message for the forceError command
	_message=`/bin/echo "$_message" | /usr/bin/tr -s " " " "`
	_cmd=`commandPath exec_diag`
	if [ $MultiBlade = TRUE ]
	then
		$_cmd forceError --slot $Slot -severity 4 -message '"'" $Slot $MyName: $_message"'"'
	else
		$_cmd forceError --slot $Slot -severity 4 -message '"'" $MyName: $_message"'"'
	fi
	burninMessage "$_message"
	if [ "$label" != "-" ]
	then
		burninMessage "Label $label"
	fi
	/bin/sync

	#
	# Cleanup anything in the script proper
	#
	my_local_cleanup

	#
	# Restore LED control back to EM
	#
	_cmd=`commandPath exec_diag`
	if [ $MultiBlade = FALSE ]
	then
		if [ $CumulativeStatus = 0 ]
		then
			eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_FAST_BLINK > /dev/null 2>/dev/null &
		fi
		_cmd=`commandPath switchbeacon`
		$_cmd 1
		
	else
		eval $_cmd bogus_test_to_set_led --slot $Slot --setled $LED_DEFAULT > /dev/null 2>/dev/null &
		_cmd=`commandPath bladebeacon`
		$_cmd $Slot 1
	fi
}

##################################
# NAME
#    saveBladeParams - Saves the burnin parameters into non-volitile storage
#
# SYNOPSIS
#    saveBladeParams
#
# DESCRIPTION
#    save all parameters to the serial eeprom 
#    Variables stores in order are:
#
#    1.  State
#    2.  FileName
#    3.  RunNumber
#    4.  RunStatus
#    5.  CumulativeStatus
#    6.  StartTime
#    7.  RunTime
#    8.  TotalThermal
#    9.  TotalVib
#    10. TotalCmdNumber
#
#    These values are stored as a space seperated string. In this form 
#    they can be parsed with the /usr/bin/cut -d" " command to get the individual
#    values when read.
#
# ALSO SEE
#    getBladeParam
#
# OPERANDS
#    NONE
##################################

saveBladeParams()
{
	_cmd=`commandPath saveBladeParameters`
	$_cmd -slot $1 -offset 0 -data "ST:$State SN:$FileName RN:$RunNumber RS:$RunStatus CS:$CumulativeStatus TS:$StartTime TR:$RunTime TT:$TotalThermal TV:$TotalVib TC:$TotalCmdNumber CN:$CmdNumber"
	if [ "$debug_on" != "0" ]
	then
		if [ "$LogFile" != "" ]
		then
			$LogCmd $LogFile "DEBUG saveparam: state $State filename $FileName runnumber $RunNumber runstatus $RunStatus cumstat $CumulativeStatus starttime $StartTime runtime $RunTime totaltime $TotalThermal totalvib $TotalVib totalcmd $TotalCmdNumber cmdnumber $CmdNumber"
		fi
	fi
}

##################################
# NAME
#    getBladeParam - returns a burnin parameter from non-volitile storage
#
# SYNOPSIS
#    getBladeParam keyword
#
# DESCRIPTION
#    Get a specific data field from the seeprom. The field to keyword
#    mapping is as follows:
#
#    1.  STATE            State
#    2.  SCRIPT_NAME      FileName
#    3.  RUN_NUMBER       RunNumber
#    4.  RUN_STATUS       RunStatus
#    5.  CUM_STATUS       CumulativeStatus
#    6.  START_TIME       StartTime
#    7.  RUN_TIME         RunTime
#    8.  TEMP_CYCLE       TotalThermal
#    9.  VIB_CYCLE        TotalVib
#    10. TOTAL_CMD_NUMBER TotalCmdNumber
#    11. CMD_NUMBER       CmdNumber
#
#    The entire value string is extracrted from non-volitile storage,
#    and parsed for the specified field value. If the keyword is not
#    one of the eleven above, the string invalid parameter is returned
#
#    This function works in concert with saveBladeParams().
#
# OPERANDS
#    $1 slot        Slot to access
#    $2 keyword     Named field of a value desired
#
##################################
getBladeParam()
{
	gBP_result=0
	_cmd=`commandPath getBladeParameters`
	BladeParameters=`$_cmd -slot $1`
	if [ "$debug_on" != "0" ]
	then
		if [ "$LogFile" != "" ]
		then
			$LogCmd $LogFile "DEBUG getparam - $BladeParameters"
		fi
	fi

	#
	# Start by looking in the default position for the data.
	#
	case $2 in
	"STATE")
		fld_str="ST"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f1`
		;;
	"SCRIPT_NAME")
		fld_str="SN"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f2`
		;;
	"RUN_NUMBER")
		fld_str="RN"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f3`
		;;
	"RUN_STATUS")
		fld_str="RS"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f4`
		;;
	"CUM_STATUS")
		fld_str="CS"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f5`
		;;
	"START_TIME")
		fld_str="TS"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f6`
		;;
	"RUN_TIME")
		fld_str="TR"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f7`
		;;
	"TEMP_CYCLE")
		fld_str="TT"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f8`
		;;
	"VIB_CYCLE")
		fld_str="TV"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f9`
		;;
	"TOTAL_CMD_NUMBER")
		fld_str="TC"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f10`
		;;
	"CMD_NUMBER")
		fld_str="CN"
		fld_result=`/bin/echo "$BladeParameters" | /bin/grep $fld_str | /usr/bin/cut -d" " -f11`
		;;
	*)
		fld_str="XX"
		gBP_result=fld_result="invalid parameter"
		;;
	esac

	#
	# Default the field if it is missing
	#
	if [ "$fld_result" = "" ]
	then
		$LogCmd $LogFile "$slot_message $MyName: Missing the $fld_str data"
		case $fld_str in
		"ST")
			gBP_result=ACTIVE;;
		"SN")
			gBP_result=`burninSetScript -show`;;
		"RN")
			gBP_result=1;;
		"RS")
			gBP_result=0;;
		"C")
			gBP_result=0;;
		"TS")
			gBP_result=`/bin/date \+\%s`;;
		"TR")
			gBP_result=`/bin/date \+\%s`;;
		"TT")
			gBP_result=0;;
		"TV")
			gBP_result=0;;
		"TC")
			gBP_result=0;;
		"CN")
			gBP_result=1;;
		"XX")
			gBP_result="invalid"
		esac
	else
		#
		# Make sure the data is the correct field requested.
		#
		field=`/bin/echo $fld_result | /usr/bin/cut -d":" -f1 | cut -d" " -f2`
		if [ "$field" = "$fld_str" ]
		then
			# We have the correct information, so extract the value.
			gBP_result=`/bin/echo $fld_result | /usr/bin/cut -d":" -f2`
		else
			#
			# Determine how many fields there are.
			#
			max_field=`/bin/echo $BladeParameters | wc -w | sed "s/ //g"`
			if [ "$max_field" != "11" ]
			then
				$LogCmd $LogFile "$slot_message $MyName: incorrect bladeparameters: $BladeParameters."
			fi
			_pos=1
			while [ $_pos -le $max_field ]
			do
				field=`/bin/echo $BladeParameters | /usr/bin/cut -d":" -f$_pos | /usr/bin/cut -d" " -f2`
				if [ "$field" = "$fld_str" ]
				then
					gBP_result=`/bin/echo $BladeParameters | /usr/bin/cut -d":" -f$_pos | /usr/bin/cut -d" " -f1`
					break
				fi
			_pos=`expr $_pos + 1`
			done
		fi
	fi
	/bin/echo "$gBP_result"
}

##################################
# NAME
#    burninSetName - Set the diagnostics burnin test name.
#
# SYNOPSIS
#    burninSetName name | "-show" slot [ -noupdate ]
#
# DESCRIPTION
#    This command sets the name of the current burnin test.  This value
#    is used in messages to indicate which test was running.
#
#    The mode becomes active as soon as this command is executed.
#    It does not require a reboot to take effect.
#
# OPERANDS
#    The following operands are optional:
#
#       name	<blank>, "-show": The current burnin name will
#		be displayed.  Otherwise the burnin name will
#		be set to the specified string.
#
#       slot	Which slot to update.
#
#   -noupdate  Inhibits the configuration update upon succesfull
#              completion of the script
#
##################################
burninSetName()
{
	_x=`/usr/bin/expr "$2" : "\([0-9]\+\)"`
	if { [ -z "$_x" ] || [ $_x -lt $_def_slot ] || [ $_x -gt 100 ] ; }
	then
		_slot=$_def_slot
	else
	        _slot=$_x
	fi

	#
	# Program customization
	#

	_config_string="diag.mode.burnin.$_slot.name" # config string to update
	if [ `isMultiBlade` = TRUE ]
	then
		_config_name="Slot $_slot burnin name"	# User name of config.
	else
		_config_name="burnin name"	# User name of config.
	fi
	_config_default=""		# default value
	_config_mode=$STRING		# config mode

	#
	# burninname()
	#
	checkForShow "$_config_name" "$_config_string" "$_config_mode" "$_config_default" $1

	setConfig $_config_string $_config_mode "$1"
	/bin/echo "$_config_name is now $1"

	if [ "$3" != "-noupdate" ]
	then
		updateConfig
	fi
}

##################################
# NAME
#    burninSetLevel - Set the diagnostics burnin level.
#
# SYNOPSIS
#    burninSetLevel [ level | "-show" ] [ -noupdate ]
#
# DESCRIPTION
#    This command selects or displays the burnin level.  When the
#    burnin level is set to a value other than 0, the diagnostics
#    daemon program will perform burnin testing in place of POST 2
#    each time a switch blade is powered on.
#
#    The mode becomes active as soon as this command is executed.
#    It does not require a reboot to take effect.
#
#    When a burnin level other than 0 is selected, the actual behavior
#    is determined by the configuration of the diagnostics daemon and
#    the burnin scripts that are run.
#
#    Note:  This command sets both diag.mode.burnin.level and
#    diag.mode.burnin.
#
# OPERANDS
#    The following operand is optional:
#
#       level		Integer value or "-show".  If level
#			is missing or the string "-show" then
#			the current burnin level will be displayed.
#			If an integer is provided then the burnin
#			level will be set to that value.
#
#                       If no parameter is provided the current
#			debug level setting will be displayed.
#
#   -noupdate  Inhibits the configuration update upon succesfull
#              completion of the script
#
#       -noupdate      
#
# SEE ALSO
#     diagsetburnin, diagresetburnin, setburninscript
#
##################################
burninSetLevel()
{
	#
	# Program customization
	#
	_config_string="diag.mode.burnin.level" # config string to update
	_config_string2="diag.mode.burnin"			# config string to update
	_config_name="Burnin level"	# User name of config.
	_config_default=0		# default value
	_config_mode=$INTEGER		# config mode


	########################
	#
	# burninlevel()
	#

	checkForShow "$_config_name" "$_config_string" "$_config_mode" "$_config_default" $1
	_val=`getValue $1`
	if [ $? != 0 ] ; then exit 3 ; fi

	if [ $_val -lt 0 ] ; then
	err "Burnin level must be >= 0"
	exit 3
	fi

	# set level
	setConfig $_config_string $_config_mode $_val
	# /bin/echo $_config_name is now $_val.

	# set enable
	if [ $_val != 0 ] ; then
	_val=1
	fi
	setConfig $_config_string2 $_config_mode $_val
	if [ "$_val" = "1" ] ; then
	_res=Enabled
	else
	_res=Disabled
	fi
	/bin/echo Burnin mode is $_res.
	if [ "$2" != "-noupdate" ]
	then
		updateConfig
	fi
}
##################################
# NAME
#    burninSetScript - Set diagnostics post/burnin scripts.
#
# SYNOPSIS
#    burninSetScript type event [ path ] [ -noupdate ]
#
# DESCRIPTION
#    This command selects the scripts to be run by the diagnostics
#    daemon for various events.  When the specified event occurs for
#    the specified board type then the script will be run.
#
#
#
#    The mode becomes active as soon as this command is executed.
#    It does not require a reboot to take effect.
#
# OPERANDS
#
#    type      Board type.  Possible values include "CP", "GP", "WAN",
#	       and "SWITCH".  For normal operations on switch blades
#	       "SWITCH" should be specified.
#
#    event     Event type.  Valid settings are "POST1", "POST2" and
#	       "BURNIN"
#
#    path      Path name for script to run.  If this parameter is ommited
#	       or contains the string "-show" then the current setting will
#	       be displayed.
#
#   -noupdate  Inhibits the configuration update upon succesfull
#              completion of the script
#
#
# Program customization
#
##################################
burninSetScript()
{
	_typ=$1
	_event=$2
	_config_string='diag.script.$_typ.$_event' # config string to update
	_config_name='$_event script for $_typ boards'	# User name of config.
	_config_default="-none-"		# default value
	_config_mode=$STRING		# config mode

	#
	# burninscript
	#

	_config_string=`eval /bin/echo $_config_string`
	_config_name=`eval /bin/echo $_config_name`
	checkForShow "$_config_name" "$_config_string" "$_config_mode" "$_config_default" "$3"

	#
	# Make sure file exists
	#

	_ok=0
	for _f in "$3" "$FABOSHOME/share/$3" "$FABOSHOME/sbin/$3" "$home/$3"; do
	if [ -x $_f ] ; then
		_ok=1
		break;
	fi
	done

	if [ $_ok = 0 ] ; then
	   err "$3 is not executable"
	   exit 3
	fi

	#
	# Make sure this is good setting
	#
	_ok=`eval testConfig $_config_string $_config_mode`
	if [ $? != 0 ] ; then exit 3 ; fi
	if [ $_ok = 0 ] ; then
	   /bin/echo "$config_string is not in database."
	   /bin/echo "Are you sure [y/n d:n] ?"
	read _ans
	if [ "$_ans" != "y" ] ; then exit 3 ; fi
	fi

	setConfig $_config_string $_config_mode $_f
	/bin/echo $_config_name is now $_f
	if [ "$4" != "-noupdate" ]
	then
		updateConfig
	fi
}
##################################
# NAME
#    getNumAvailableScripts - Output the number of scripts that
#                             contain the specified keyword
#
# SYNOPSIS
#    getNumAvailableScripts Keyword [-config | -noconfig ]
#
# DESCRIPTION
#    Output the number of scripts available that support the specified
#    keyword activity. Include the current script if there has been
#    one selected.
#
#    The determination is triggered by the presence of the keyword
#    $1 in the files in $FABOSHOME/share. If there is a variable that
#    holds the current selection, then $2 contains "-config". If there
#    is no config variable then $2 = "-noconfig"
#
# OPERANDS
#    $1 - Keyword to search for
#    $2 - "-config" "-noconfig"
##################################
getNumAvailableScripts()
{
  if [ "$2" = "-config" ]
  then
    /bin/echo "-current" > /tmp/$$.tmp
  fi
  /bin/grep "$1" /fabos/share/* 2>/dev/null | \
      /usr/bin/cut -d":" -f1 | \
      /bin/sort -u | \
      /bin/grep -v diagcommon | \
      /bin/sed "s/\/fabos\/share\///g" >> /tmp/$$.tmp

  /usr/bin/wc -l /tmp/$$.tmp | \
		  /usr/bin/tr -s " " " " | \
		  /usr/bin/cut -d" " -f2
  /bin/rm -f /tmp/$$.tmp > /dev/null 2>&1
}

##################################
# NAME
#    showAvailableScripts - Output the names of scripts that
#                           contain the specified keyword
#
# SYNOPSIS
#    showAvailableScripts Keyword [ -noconfig | configvar default ]
#
# DESCRIPTION
#    Output a list of the scripts available that support burnin activity.
#    Include the current burnin script if there has been one selected.
#
#    The determination is triggered by the presence of the keyword
#    $1 in the files in $FABOSHOME/share. If there is a variable that
#    holds the current selection, then the variable name is specified
#    in $2 and the default value in $3. The config variables are
#    assumed to be of type $STRING. 
#
#    If there is no config variable then $2 = "-noconfig"
#
# OPERANDS
#    $1 - Keyword to search for
#    $2 - Config variable that contains the current setting 
#	  (if none then "-noconfig")
#    $3 - Defaults value for config variable
##################################
showAvailableScripts()
{
  # Add the EXIT choice
  /bin/echo "EXIT" > /tmp/$$.tmp

  # Add current setting if available
  if [ "$2" = "-noconfig" ]
  then
    _script=""
  else
    _script=`getConfig "$2" $STRING $3`
  fi

  if [ ! -z "$_script" ]
  then
    /bin/echo "-current \($_script\)" >> /tmp/$$.tmp
  fi

  # generate list of remaining choices
  /bin/grep "$1" /fabos/share/* 2>/dev/null | \
      /usr/bin/cut -d":" -f1 | \
      /bin/sort -u | \
      /bin/grep -v diagcommon | \
      /bin/sed "s/\/fabos\/share\///g" >> /tmp/$$.tmp

  # format output as " #) scriptname"
  _num=0
  /bin/cat /tmp/$$.tmp | while read _line
  do
    /bin/echo " $_num) $_line"
    _num=`/usr/bin/expr $_num + 1`
  done
  /bin/rm -f /tmp/$$.tmp > /dev/null 2>&1
}

##################################
# NAME
#    getScriptSelection - Output the name of nth script that
#                         contain the specified keyword
#
# SYNOPSIS
#    getScriptSelection N Keyword [ -noconfig | configvar default ]
#
# DESCRIPTION
#    Output the Nth script available that supports keyworded activity.
#    Include the current burnin script if there has been one selected.
#
#    The determination is triggered by the presence of the keyword
#    $1 in the files in $FABOSHOME/share. If there is a variable that
#    holds the current selection, then the variable name is specified
#    in $2 and the default value in $3. The config variables are
#    assumed to be of type $STRING. 
#
#    If there is no config variable then $2 = "-noconfig"
#
# OPERANDS
#    $1 - Nth selection
#    $2 - Keyword to search for
#    $3 - Config variable that contains the current setting 
#	  (if none then "-noconfig")
#    $4 - Defaults value for config variable
##################################
getScriptSelection()
{
  # Add the EXIT choice
  /bin/echo "EXIT" > /tmp/$$.tmp

  # generate list of script names. Include current if specified
  if [ "$3" = "-noconfig" ]
  then
    _script=""
  else
    _script=`getConfig "$3" $STRING $4`
  fi
  if [ "$_script" != "" ]
  then
    /bin/echo "$_script" >> /tmp/$$.tmp
  fi
  /bin/grep "$2" /fabos/share/* 2>/dev/null | \
      /usr/bin/cut -d":" -f1 | \
      /bin/sort -u | \
      /bin/grep -v diagcommon | \
      /bin/sed "s/\/fabos\/share\///g" >> /tmp/$$.tmp

  # get the max allowable value for N
  _num_choice=`/usr/bin/wc -l /tmp/$$.tmp | \
			   /usr/bin/tr -s " " " " | \
			   /usr/bin/cut -d" " -f2`

  # get the selected choice
  if [ $1 -lt 0 -o $1 -gt $_num_choice ]
  then
    /bin/echo "Invalid"
  else
    _num=0
    /bin/cat /tmp/$$.tmp | while read _line
    do
      if [ $_num -eq $1 ]
      then
        /bin/echo "$_line"
        break
      fi
      _num=`/usr/bin/expr $_num + 1`
    done
  fi
#  /bin/rm -f /tmp/$$.tmp > /dev/null 2>&1
}


